// ReaderWriter.h
// Definiert die Klassen Database, Reader und Writer zur 
// Lsung des Reader-Writer Problems
// -----------------------------------------------------------
#include <thread>
#include <mutex>
#include <condition_variable>
#include <string>
#include <iostream>
using namespace std;

class Database
{
   private:
      string data;                    // Daten 

      int cntWriter;                  // Anzahl Writer
      int cntReader;                  // Anzahl Reader

      bool busyWriter;                // busyWriter-Flag
      bool busyReader;                // busyReader-Flag

      mutex mtx;

      condition_variable okToRead;
      condition_variable okToWrite;

   public:
     Database() : cntWriter(0), cntReader(0), 
                  busyWriter(false), busyReader(false){}  

     void write(const string& s)
     {
        unique_lock<mutex> lock(mtx);

        cntWriter++;
        while(busyWriter || busyReader)   // Falls anderer 
           okToWrite.wait(lock);          // Thread aktiv: 
                                          // warten. 
        busyWriter = true;
        data += s;

        cout << "\nGeschrieben: " << data << endl;
        cout << "Gesamtzahl Writer:" << cntWriter << endl;

        cntWriter--;
        busyWriter = false;

        okToWrite.notify_one();     // Blockierungen aufheben
        okToRead.notify_all();  
     }

     void read()
     {
        unique_lock<mutex> lock(mtx);

        cntReader++;                       // Solange anderer
        while (busyWriter || data == "")   // Writer aktiv 
           okToRead.wait(lock);            // oder nichts zu 
                                           // lesen: warten. 
        busyReader = true;

        cout << "\nGelesen: " << data << endl;
        cout << "Gesamtzahl Reader:" << cntReader << endl;

        cntReader--;
        busyReader = false;

        okToWrite.notify_one();     // Blockierung aufheben
    }
};

class Writer
{
   private:
     Database& dataref;
   public:
     Writer(Database& b): dataref(b){ }

     void operator()(const string& s)
     {
        dataref.write(s);
     }
};


class Reader
{
   private:
     Database& dataref;

   public:
     Reader(Database& b) : dataref(b){}

     void operator()()
     {
        dataref.read();
     }
};
