// mySmartPtr_t.cpp
// Test der eigenen Klasse SmartPtr<T>
// ---------------------------------------------------------
#include "mySmartPtr.h"
#include <iostream>
using namespace std;

struct Base
{
    virtual void hello() 
    { cout << "Hier ist die Klasse Base" << endl; }
    virtual ~Base() { cout << "Base-Destruktor" << endl; }
};

struct Derived : public Base 
{
    virtual void hello() 
    { cout << "Hier ist die Klasse Derived" << endl; }
    virtual ~Derived() { cout << "Derived-Destruktor" << endl; }
};

// bergabe per value:
void func(SmartPtr<Base> p)
{
    cout << "In der Funktion func(): ";
    p->hello();         // (p.operator->())->hello();
}

int main()
{
    cout << "--- Smart Pointer auf dynamische Objekte ---" 
         << endl;
    SmartPtr<Base> sptrB(new Base);    // Mit Initialisierung
    SmartPtr<Derived> sptrD;           // Zeigt auf nichts

    sptrD = (SmartPtr<Derived>)new Derived;  // Zuweisung eines
                                             // temporren Pointers.
    cout << "\nOperator ->" << endl;
    sptrB->hello();            // (sptrB.operator->())->hello();
    sptrD->hello();

    cout << "\nOperator *" << endl;
    (*sptrB).hello();
    (*sptrD).hello();

    cout << "\nKopierkonstruktor und Zuweisung" << endl;
    if( sptrB)                          // wahr, da sptrB != 0.
    {
      //  Initialisierung mit Kopierkonstruktor
      SmartPtr<Base> sptrB2(sptrB);     // 2. Besitzer von *sptrB

      //  Zuweisung von Smart-Pointern:
      SmartPtr<Base> sptrB3;            // leer
      sptrB3 = sptrB;                   // 3. Besitzer von *sptrB
      cout << "Anzahl Besitzer von Base-Objekt:    "
           << *(sptrB.getCountPtr()) << endl;  // 3
    }                                   // Jetzt nur noch ein Besitzer

    cout << "\nPolymorphismus:" << endl;
    // Parameterbergabe eines smartPointer
    func(sptrB);                 // Call by Value.
    func(sptrD);                 // Mit Konvertierung in Base-Ptr.

    // Basisklassenzeiger auf ein Derived-Objekt zeigen lassen:
    sptrB = sptrD;               // Base-Objekt wird zerstrt.
                                 // Jetzt zwei Besitzer von *sptrD
    cout << "Anzahl Besitzer des Derived-Objekts: "
         << *(sptrD.getCountPtr()) << endl;

    sptrD.reset();                      // Noch ein Besitzer. 
    sptrB->hello();                     // Derived::hello();

    SmartPtr<Base> sptrB2(new Base);    // Neues Base-Objekt.
    sptrB.swap( sptrB2); 
    sptrB->hello();                     // Base::hello();
    sptrB2->hello();                    // Derived::hello();

    cout << "\nEnde von main()!" << endl;
    return 0;
}
