// inserter.cpp: Algorithmen mit Insert-Iteratoren aufrufen.
// Liest Gleitpunktzahlen aus einer Datei und verarbeitet diese.
// ---------------------------------------------------------------
#include <iostream>
#include <fstream>         // Die File-Stream-Klassen
#include <vector>          // Definition des vector-Templates
#include <set>             // Definition des set-/multiset-Templates
#include <algorithm>       // Definition der Algorithmen-Templates
#include <iterator>        // Definition vordefinierter Iteratoren
using namespace std;

// Zur Anzeige von float-Werten
void print( float x) {  cout.width(8);  cout << x; }

char filename[] = "data.txt";           // Datei mit float-Werten
vector<float> flArr;                    // float-Vektor
multiset<float> flSet;                  // Multiset fr float-Werte.

int main()
{
    ifstream inFile(filename);              // Filestream anlegen
    if( !inFile)
    {  cerr << "Fehler beim Oeffnen der Datei " << filename << endl;
       return 1;
    }
    // Daten der Datei in einen Vektor einlesen:
    copy( istream_iterator<float>(inFile),  // Input-Iterator
          istream_iterator<float>(),        // EOF-Iterator  
          back_inserter(flArr));            // Back-Inserter

    if( !inFile.eof())
       cerr << "Fehler beim Lesen der Datei " << filename << endl;
    inFile.close();                         // Datei schlieen

    if( flArr.size() == 0)
    {  cerr << "Keine Daten vorhanden!" << endl;
       return 2;
    }

    // Vektor anzeigen:
    cout << "Die eingelesenen Werte:" << endl;
    for_each( flArr.begin(), flArr.end(), print);

    // Vektor mit Reverse-Iterator durchlaufen:
    cout << "\n... und in umgekehrter Reihenfolge:" << endl;
    for_each( flArr.rbegin(), flArr.rend(), print);

    // Werte in den Set-Container einfgen. 
    // Dadurch werden die Werte auch sortiert:
    copy( flArr.begin(), flArr.end(),             // Quell-Bereich
          inserter(flSet, flSet.begin()) );       // Ziel

    cout << "\n... und jetzt sortiert:" << endl;
    for_each( flSet.begin(), flSet.end(), print);
    cout << endl;

    return 0;
}

/* Beispiel-Ausgabe:

Die eingelesenen Werte:
    11.1     2.2    33.3   -4.44     5.5
... und in umgekehrter Reihenfolge:
     5.5   -4.44    33.3     2.2    11.1
... und jetzt sortiert:
   -4.44     2.2     5.5    11.1    33.3

*/
