// fourier.cpp: Definiert die Template-Funktion 
// Fourier() zur Berechnung der diskreten Fourier-
// Tranformierten fr die Koeffizienten 
// (a[0], ..., a[n-1]), wobei n eine Potenz von 2 ist. 
// ----------------------------------------------------
#include <complex>
#include <valarray>
#include <numbers>             // Konstante pi (C++20)
using namespace std::numbers;  // im namespace numbers.
using namespace std;

//#define pi 3.1415927
typedef complex<double> COMPLEX;

valarray<COMPLEX> Fourier(const valarray<COMPLEX>& a)
{
   int n = (int)a.size();
   if(n <= 1) 
      return a;
   else
   {
     COMPLEX wn(cos(2*pi/n), sin(2*pi/n));
     COMPLEX w = COMPLEX(1,0);
                 // Teilvektor aus geraden Indizes:
     valarray<COMPLEX> a0 = a[slice(0, n/2, 2)];  
                 // Teilvektor aus ungeraden Indizes:
     valarray<COMPLEX> a1 = a[slice(1, n/2, 2)];  
                 // Rekursiver Aufruf fr 1. Teilvektor:
     valarray<COMPLEX> y0 = Fourier(a0);         
                 // Rekursiver Aufruf fr 2. Teilvektor:
     valarray<COMPLEX> y1 = Fourier(a1);         
                 // Ergebnisse zusammenfgen:
     valarray<COMPLEX> y(n);
     for(int k=0; k <= n/2 - 1; k++)
     {
        y[k]     = y0[k] + w * y1[k];
        y[k+n/2] = y0[k] - w * y1[k];
        w *= wn;	
     }
     return y;
   }
}

#include <iostream>

int main()
{
    valarray<COMPLEX> a{ 1.0, -1.0, 0.0, 2.0 };
    valarray<COMPLEX> ft = Fourier(a);

    for( size_t i = 0; i < ft.size(); ++i)
       cout << ft[i] << " ";
    cout << endl;

    return 0;
}
