// -----------------------------------------------------
// datum.h 
// Definition der Klasse Datum mit verbesserter 
// Funktionalitt, z.B. Bereichsprfung.
// -----------------------------------------------------

#ifndef DATUM_H   // Mehrfaches Inkludieren verhindern.
#define DATUM_H

#include <string>
using namespace std;

class Datum
{
  private:
    short tag, monat, jahr;

  public:               
    Datum()                        // Default-Konstruktor
    {
       tag = monat = jahr = 1;
    }
    Datum( int tag, int monat, int jahr)
    {
       if( !setDatum( tag, monat, jahr) )
         tag = monat = jahr = 1;   // Falls Datum ungltig.
    }

    void setDatum();           // Setzt das aktuelle Datum.
    bool setDatum( int tag, int monat, int jahr);

    int getTag()   const { return tag;   }
    int getMonat() const { return monat; }
    int getJahr()  const { return jahr;  }

    bool isEqual( const Datum& d) const
    {
       return  tag == d.tag && monat == d.monat
                            && jahr  == d.jahr ;
    }
    bool isLess( const Datum& d) const;
    
    const string& asString() const;
    void print(void) const;
};

inline bool Datum::isLess( const Datum& d) const
{
    if( jahr != d.jahr)         return jahr < d.jahr;
    else if( monat != d.monat)  return monat < d.monat;
    else                        return tag < d.tag;
}

inline bool isLeapYear( int jahr)
{
    return (jahr%4 == 0  &&  jahr%100 != 0) || jahr%400 == 0;
}

#endif   //  DATUM_H
