// --------------------------------------------------------
// TelList.cpp 
// Implementierung der TelList-Methoden.
// --------------------------------------------------------

#include "telList.h"      // Definition der Klasse TelList
#include <iostream>
#include <iomanip>
#include <fstream>
using namespace std;

bool TelList::append( const string& name,      // Anhngen
                      const string& telNr)
{
    if( count < MAX                 // Falls noch Platz,
        && name.length() > 1        // mindestens 2 Zeichen,
        && search(name) == PSEUDO)  // noch nicht vorhanden
    {
      v[count].name  = name;
      v[count].telNr = telNr;
      ++count;
      dirty = true;
      return true;
    }
    return false;
}

bool TelList::erase( const string& key )       // Lschen
{
   int i = search(key);
   if( i != PSEUDO )
   {   
     if( i != count-1)            // Letztes Element in
         v[i] = v[count-1];       // Position i kopieren.
     --count;
     dirty = true;
     return true;
   }
   return false;
}

int TelList::search( const string& key ) const  // Suchen
{
   for( int i = 0; i < count; i++ ) // Suchen.
     if( v[i].name == key )
       return i;                // Gefunden

   return PSEUDO;               // Nicht gefunden
}

// Hilfsfunktionen fr die Ausgabe:
inline void tabHeader()         // berschrift der Tabelle
{
   cout << "\n  Name                        Telefon-Nr\n"
           "----------------------------------------------"
        << endl;
}
inline void printline( const Element& el)    // Eine Zeile
{
   cout << left << setw(30) << el.name << el.telNr << endl;
}

void TelList::print() const     // Alle Eintrge ausgeben
{
   if( count == 0)
      cout << "\nDie Telefon-Liste ist leer!" << endl;
   else
   {
      tabHeader(); 
      for( int i = 0; i < count; ++i)
          printline( v[i]);
   }
}

// Eintrge, die mit einer bestimmten Zeichenfolge beginnen:
int TelList::print( const string& name) const
{
   int matches = 0, len = name.length();
   
   for( int i = 0; i < count; ++i)
   {
      if( v[i].name.compare(0, len, name) == 0)
      {
        if( matches == 0) tabHeader();  // berschrift vor
                                        // erster Ausgabe. 
        ++matches;
        printline( v[i]);
      }
   }
   if( matches == 0)
      cout << "Kein passender Eintrag gefunden!" << endl;

   return matches;
}

int TelList::getNewEntries()            // Neue Eintrge
{                                       // im Dialog einlesen.
    int inputCount = 0;

    cout << "\nNeue Namen und Telefonnummern eingeben:"
            "\nEnde mit leerer Eingabe"  
         << endl;
    Element el;
    while( true)
    {
      cout << "\nNeuer Nachname, Vorname:  "; 
      getline( cin, el.name);
      if( el.name.empty())
        break;
      cout << "\nTelefonnummer: "; 
      getline( cin, el.telNr);

      if( !append( el))
      {
         cout << "Name wurde nicht eingefuegt!" << endl;
         if( count == MAX)
         {
            cout << "Die Tabelle ist voll!" << endl;
            break;
         }
         if( search( el.name) != PSEUDO)
            cout << "Name schon vorhanden!" << endl;
      }
      else
      {
         ++inputCount;
         cout << "Neues Element wurde eingefuegt!" << endl;
      }
    }
    return inputCount;
}

// Methoden zum Laden und Speichern der Telefon-Liste.

bool TelList::load()
{
   cout << "\n--- Telefon-Liste aus einer Datei laden. ---\n"
        << "\nDatei: ";
   
   string file;
   getline( cin, file);
   if( file.empty())
   {
      cerr << "Kein Dateiname angegeben!" << endl;
      return false;
   }

   // Datei zum Lesen oeffnen:
   ifstream infile( file, ios::in | ios::binary);
   if( !infile )
   {
      cerr << "Datei " << file 
           << " konnte nicht geoeffnet werden!" << endl;
      return false;
   }

   int i = 0;
   while( i < MAX)
   {
      getline( infile, v[i].name, '\0');
      getline( infile, v[i].telNr, '\0');
      if( !infile)
         break;
      else
         ++i;
   }
   if( i == MAX)
      cerr << "Obergrenze von " << MAX
           << " Eintraegen erreicht!" << endl;
   else if( !infile.eof())
   {
      cerr << "Fehler beim Lesen der Datei " << file 
           << endl;
      return false;
   }
   
   count = i;
   filename = file;
   dirty = false;

   return true;
}

bool TelList::saveAs()
{
   cout << "\n--- Telefon-Liste in einer Datei speichern. ---\n"
        << "\nDatei: ";
   
   string file;
   getline( cin, file);
   if( !setFilename(file) )
   {
      cerr << "Kein Dateiname angegeben!" << endl;
      return false;
   }
   else
   {
      return save();
   }
}

bool TelList::save()
{
   if( filename.empty())
       return saveAs();

   if( !dirty)
       return true;

   ofstream outfile( filename, ios::out | ios::binary);
   if( !outfile )
   {
      cerr << "Datei " << filename 
           << " konnte nicht geoeffnet werden!" << endl;
      return false;
   }
   
   int i = 0;
   while( i < count)
   {
      outfile << v[i].name  << '\0';
      outfile << v[i].telNr << '\0';
      if( !outfile)
         break;
      else
         ++i;
   }
   if( i < count)
   {
      cerr << "Fehler beim Schreiben der Datei " << filename 
           << endl;
      return false;
   }
   dirty = false;
   return true;
}
