// ----------------------------------------------------
// DayTime.h
// Die Klasse stellt die Uhrzeit mit Stunden, Minuten 
// und Sekunden dar.
// Die Klasse DayTime mit allen Vergleichs-Operatoren,
// den Operatoren ++ und -- (Prfix und Postfix),
// sowie den Operatoren << und >> fr Ein- und Ausgabe.
// ----------------------------------------------------
#ifndef DAYTIME_H
#define DAYTIME_H

#include <iostream>
#include <iomanip>
#include <compare>      // C++20: Fr den Operator <=> 
using namespace std;
 
class DayTime
{
  private:
    short hour, minute, second;
    bool overflow, underflow;

    void inc()            // Private Hilfsfunktion fr ++
    {
      ++second;
      if( second >= 60)         // berlauf behandeln. 
          second = 0,  ++minute;
      if( minute >= 60)
          minute = 0,  ++hour;
      if( hour >= 24)
          hour = 0,  overflow = true;
    }
    void dec()            // Private Hilfsfunktion fr --
    {
      --second;
      if( second < 0)          // Unterlauf behandeln. 
          second = 59,  --minute;
      if( minute < 0)
          minute = 59,  --hour;
      if( hour < 0)
          hour = 0,  underflow = true;
    }

  public:
    DayTime( int h = 0, int m = 0, int s = 0)
    { 
      overflow = underflow = false;
      if( !setTime( h, m, s)) 
        hour = minute = second = 0;
    }
    bool setTime(int hour, int minute, int second = 0)
    {
      if(    hour   >= 0  &&  hour < 24 
          && minute >= 0  &&  minute < 60
          && second >= 0  &&  second < 60 ) 
      {
        this->hour   = (short)hour; 
        this->minute = (short)minute; 
        this->second = (short)second; 
        return true;
      }
      else
        return false;
    }
    int getHour()   const { return hour;   }
    int getMinute() const { return minute; };
    int getSecond() const { return second; };

    int asSeconds() const    // Tageszeit als Sekunden
    {
      return (60*60*hour + 60*minute + second);
    } 

    DayTime& operator++()          // ++Sekunden
    {
      inc();
      return  *this;
    }
    DayTime operator++(int)        // Sekunden++
    {
      DayTime temp(*this);
      inc();
      return temp;
    }

    DayTime& operator--()          // --Sekunden
    {
      dec();
      return  *this;
    }
    DayTime operator--(int)        // Sekunden--
    {
      DayTime temp(*this);
      dec();
      return temp;
    }
    // Vergleichsoperatoren:
    // Der Default-Operator <=> vergleicht die Elemente 
    // der Klasse in der Reihenfolge ihrer Deklaration
    // und stoppt beim ersten ungleichen Element.
    friend auto operator<=>(const DayTime& t1, 
                            const DayTime& t2) = default;
    // Implizit sind auch <, <=, >, >=, und ==, != definiert.
};

/*
// ---  Vergleichsoperatoren  ---
// Die klassischen Vergleichsoperatoren sind explizit zu berladen,
// wenn der Default-Operator <=> nicht vorhanden ist, d.h. vor C++20.

//  t1 < t2
inline bool operator<( const DayTime& t1, const DayTime& t2)
{
   return  t1.asSeconds() < t2.asSeconds();
}

//  t1 <= t2
inline bool operator<=( const DayTime& t1, const DayTime& t2)
{
   return  t1.asSeconds() <= t2.asSeconds();
}

//  t1 == t2
inline bool operator==( const DayTime& t1, const DayTime& t2)
{
   return  t1.asSeconds() == t2.asSeconds();
}

//  t1 != t2
inline bool operator!=( const DayTime& t1, const DayTime& t2)
{
   return !(t1 == t2);
}

//  t1 > t2
inline bool operator>( const DayTime& t1, const DayTime& t2)
{
   return (t2 < t1);
} 

//  t1 >= t2
inline bool operator>=( const DayTime& t1, const DayTime& t2)
{
   return !(t1 < t2);
} 
*/
/*
// Alternative: Explizite berladung des Operators <=>.
//#include <compare>
inline strong_ordering operator<=>(const DayTime& t1, const DayTime& t2)
{
    return t1.asSeconds() <=> t2.asSeconds();
}
// Dann sind implizit auch <, <=, >, >= definiert, nicht aber == und !=
// Damit auch == und != definiert sind, muss == berladen werden.
*/

// Ein- und Ausgabe
inline ostream& operator<<( ostream& os, const DayTime& t)
{
   os << setfill('0')
      << setw(2) << t.getHour()    << ':'
      << setw(2) << t.getMinute()  << ':'
      << setw(2) << t.getSecond()  << " Uhr";
   os << setfill(' ');

   return os;
}

inline istream& operator>>( istream& is, DayTime& t)
{
   cout << "Tageszeit im Format hh:mm:ss eingeben: ";

   int std = 0, min = 0, sek = 0;
   char c1 = 0, c2 = 0;
   if( !(is >> std >> c1 >> min >> c2 >> sek)) 
      return is;
   if( c1 != ':' || c2 != ':' || ! t.setTime( std, min, sek) )
      is.setstate( ios::failbit);   // Fehler => fail-Bit setzen.
    
   return is;
}

#endif   // DAYTIME_H
