// ----------------------------------------------------
// Liste.cpp 
// Implementierung der Methoden der Klasse Liste,
// die nicht schon inline definiert sind.
// ----------------------------------------------------

#include "Liste.h"

// Kopierkonstruktor:
Liste::Liste(const Liste& src)
{
   // An leere Liste die Elemente aus src anhngen.
   first = last = NULL;
   ListEl *pEl = src.first;
   for(  ; pEl != NULL;  pEl = pEl->next )
      pushBack( pEl->datum, pEl->betrag);
}

// Destruktor der Liste:
Liste::~Liste()
{
   ListEl *pEl = first,
          *next = NULL;
   for(  ; pEl != NULL;  pEl = next)
   {
      next = pEl->next;
      delete pEl;
   }
}

// Zuweisung:
Liste& Liste::operator=( const Liste& src)
{
   // Speicher aller Elemente freigeben: 
   ListEl *pEl = first,
          *next = NULL;
   for(  ; pEl != NULL;  pEl = next)
   {
      next = pEl->next;
      delete pEl;
   }
   first = last = NULL;

   // An leere Liste die Elemente aus src anhngen.
   pEl = src.first;
   for(  ; pEl != NULL;  pEl = pEl->next )
      pushBack( pEl->datum, pEl->betrag);

   return *this;
}

// Neues Element an die Liste anhngen :
void Liste::pushBack(const Datum& d, double b)
{ 
   ListEl *pEl = new ListEl( d, b);
   if( last == NULL)        // Liste leer?
      first = last = pEl;
   else
      last->next = pEl,  last = pEl;
}

// Element am Anfang der Liste lschen.
void Liste::popFront()
{
   if( first != NULL)       // Nicht leer?
   {
     ListEl *pEl = first;   // Erstes Element sichern
     first = first->next;   // Zeiger eins weiter setzen.
     delete pEl;            // "Altes" erstes lschen
     if( first == NULL)     // Jetzt leer?
        last = NULL;
   }
}


// --- globale Funktionen zur Ausgabe ---

// Ausgabe eines Elementes:
ostream& operator<<( ostream& os, const ListEl& le)
{
    os << le.getDatum().asString() << "  Betrag: ";
    os << fixed << setprecision(2) << setw(10) << le.getBetrag();
    return os;
}

// Ausgabe der Liste:
ostream& operator<<( ostream& os, const Liste& liste)
{
   ListEl *pEl = liste.front();
   if( pEl == NULL)
       cout << "Die Liste ist leer!" << endl;

   for(  ; pEl != NULL;  pEl = pEl->getNext() )
      os << *pEl << endl;
   return os;
}
