// ----------------------------------------------------
// FloatVek.h : Darstellung dynamischer float-Vektoren.
// ----------------------------------------------------
#ifndef FLOATVEK_H
#define FLOATVEK_H

#include <iostream>
using namespace std;

class FloatVek
{
  private:
    float* vekPtr;    // Dynamisches Element
    int max;          // Maximale Anzahl, ohne
                      // erneut Speicher zu reservieren.
    int anz;          // Aktuelle Anzahl Elemente

    void expand( int neueGroesse);   // Hilfsfunktion, die
                                     // den Vektor vergrert.
  public:
    FloatVek( int n = 256 );
    FloatVek( int n, float wert);

    FloatVek(const FloatVek& src);           // Kopierkonstruktor 
    FloatVek(FloatVek&& src);                // Move-Konstruktor

    ~FloatVek();

    FloatVek& operator=( const FloatVek& );  // Zuweisung
    FloatVek& operator=(FloatVek&& src);     // Move-Zuweisung

    int  length() const { return anz; }

    float& operator[](int i);                // Index-Operator.
    float operator[](int i) const;

    // Methoden zum Anhngen eines float-Wertes 
    // oder eines float-Vektors:
    void append( float wert);
    void append( const FloatVek& v); 

    // Methoden zum Einfgen eines float-Wertes 
    // oder eines float-Vektors:
    bool insert( float wert, int pos); 
    bool insert( const FloatVek& v, int pos ); 
 
    bool remove(int pos);          // Element in Position pos lschen.

    // Einen zweiten Vektor elementweise hinzuaddieren:
    FloatVek& operator+=( const FloatVek& v2);

    void swap( FloatVek& v2)       // *this mit v2 Tauschen.
    {
       if( this != &v2 )
       {  // Datenelemente tauschen:
          float* p = vekPtr;  vekPtr = v2.vekPtr;  v2.vekPtr = p;
          int i;
          i = max;   max = v2.max;  v2.max = i;
          i = anz;   anz = v2.anz;  v2.anz = i;
       }
    }

    // Ausgabe des Vektors
    friend ostream& operator<<( ostream& os, const FloatVek& v);
};

// Globale Funktion zum Tauschen:
inline void swap( FloatVek& v1, FloatVek& v2)   // v1 mit v2 tauschen.
{   v1.swap(v2);   }

// Elementweise Addition zweier Vektoren:
FloatVek operator+( const FloatVek& v1, const FloatVek& v2);

#endif   // FLOATVEK_H
