// ----------------------------------------------------------
// sortvec.h: Das Klassen-Template SortVec zur
//            Darstellung eines sortierten Vektors
// ----------------------------------------------------------
#ifndef SORTVEC_H
#define SORTVEC_H

#include <vector>
#include <functional>  // Fr Comparator-Klasse less<T>
using namespace std;

template <class T, class Compare = less<T> >
class SortVec : public vector<T>
{
  public:
   SortVec() {}
   SortVec(int n, const T& x = T()) : vector<T>(n,x) {}

   void insert(const T& obj);        // Sortiert Einfgen
   int  search(const T& obj);        // Objekt suchen
   void merge(const SortVec& v);     // Mischen
};


// ----------------------------------------------------------
// Methode insert()
//
// Die Methode insert() fgt ein neues Objekt am Ende
// des Vektors ein und sortiert es dann ein.

template <class T, class Compare >
void SortVec<T, Compare>::insert(const T& obj)
{
   push_back(obj);                // Am Ende einfgen.

   auto pos = end()-1;            // Letzte Position

   while( pos > begin())          // Einsortieren:
   {
     --pos;
     if( Compare()(obj,*pos) )      // Tauschen:
     { auto temp = pos; *(++temp) = *pos;  *pos = obj; }
     else break;
   }
}

// ----------------------------------------------------------
// Methode search()
//
// Die Methode search() sucht ein vorgegebenes Objekt
// nach dem binren Suchverfahren im Container. Hierbei
// wird das Objekt mit dem mittleren Element im Vektor
// verglichen. Ist das Objekt kleiner, so wird in der
// linken Hlfte, andernfalls in der rechten Hlfte
// weitergesucht. Das binre Suchverfahren hat eine
// logarithmische Laufzeit und ist damit sehr schnell.
// Return-Wert: Index mit dem gefundenen Objekt
//              oder -1, falls nicht vorhanden.

template <class T, class Compare >
int SortVec<T, Compare>::search(const T& obj)
{
  int first = 0, mid, 
      last = end() - begin() - 1;

  while( first < last )
  {
     mid = (first + last + 1)/ 2;
                               // In unterer Hlfte suchen:
     if( Compare()(obj, (*this)[mid]) )
        last = mid - 1;
     else
        first = mid;           // bzw. in oberer Hlfte.
  }

  if( obj == (*this)[first] )  // Gefunden?
     return first;             // Ja.
  else
     return -1;                // Nicht gefunden.
}

// ----------------------------------------------------------
// Methode merge()
//
// Die Methode merge() mischt einen als Argument
// bergebenen sortierten Vektor in den Vektor *this.
// ---------------------------------------------------

template <class T, class Compare >
void SortVec<T,Compare>::merge(const SortVec<T,Compare>& v2)
{
  SortVec temp;                       // Temporrer Vektor
  SortVec& v1 = *this;                // 1. Vektor = *this
  unsigned int n1 = 0, n2 = 0;

  // Das jeweils kleinere Objekt in temp kopieren:
  while(n1 < v1.size() &&  n2 < v2.size())
    if( Compare()(v1[n1], v2[n2]) )   // if(v1[n1] < v2[n2])
       temp.push_back(v1[n1++]);
    else
       temp.push_back(v2[n2++]);
                                      // Rest anhngen:
  while( n1 < v1.size())
    temp.push_back(v1[n1++]);

  while( n2 < v2.size())
    temp.push_back(v2[n2++]);

  this->swap(temp);      // effektiver als *this = temp;
}
#endif  // SORTVEC
