// -----------------------------------------------------------
// hash_dat.h: Definition der Klassen HashEntry zur Darstellung
//             eines Datensatzes in der Hash-Datei und
//             HashFile zur Darstellung der Hash-Datei.
// -----------------------------------------------------------
#ifndef HASH_H_
#define HASH_H_
#define _CRT_SECURE_NO_WARNINGS    // Fr VC und strncpy()

#include <fstream>
#include <iostream>
#include <iomanip>
#include <string>
#include <cstring>
using namespace std;

#include "exceptio.h"

class HashEntry
{
    private:
      unsigned long nr;
      char name[30];

    public:
      HashEntry(unsigned long n = 0L, const string& s = "")
      {
         nr = n;
         setName(s);
      }
      unsigned long   getNr() const { return nr; }
      void   setNr(unsigned long n) { nr = n; }

      string getName() const { return name; }
      void   setName(const string& s)
      { strncpy(name, s.c_str(), 29);  name[29] = '\0'; }

      int getSize() const { return(sizeof(nr) + sizeof(name)); }

      fstream& write(fstream& fs) const;
      fstream& read(fstream& fs);

      fstream& write_at(fstream& fs, unsigned long pos) const;
      fstream& read_at(fstream& fs, unsigned long pos);

      virtual void display() const
      {
          cout << fixed << setprecision(2)
               << "----------------------------------\n"
               << "Kundennummer:        " << nr    << endl
               << "Kunde:               " << name  << endl
               << "----------------------------------"
               << endl;
          cin.get();
      }
};

class HashFile
{
   private:
     fstream f;             // File-Stream
     string  name;          // Dateiname
     unsigned long b;       // Groesse des Adressraums

   protected:
     unsigned long hash_func(unsigned long key) const
     { return key % b; }

   public:
     HashFile(const string s, unsigned long n );  // Mgliche Exception: open_error
     ~HashFile() { f.close(); }

     void insert( HashEntry& rec);                // Mgliche Exceptions: read_error, write_error
     HashEntry& retrieve( unsigned long key );    // Mgliche Exception:  read_error

     void display();
};

#endif
