// generate_remove.cpp: Elemente erzeugen und lschen.
// ---------------------------------------------------------------
#include <iostream>
#include <vector>           // Definition des vector-Templates
#include <algorithm>        // Definition der Algorithmen-Templates
#include <iterator>         // Definition vordefinierter Iteratoren
using namespace std;

class Fibonacci            // Die ersten beiden Fibonacci-Zahlen 
{  long f0, f1;            // sind 0 und 1. Jede weitere ist die 
 public:                   // Summe ihre beiden Vorgnger.
    Fibonacci() : f0(0), f1(1) {}
    long operator() ()
    {
       long f = f0 + f1;   f0 = f1;   f1 = f;
       return f;
    }
};

void print( long elm) {  cout.width(8);  cout << elm; }

bool even( long elm) { return elm % 2 == 0; }
bool odd( long elm)  { return elm % 2 != 0; }

int main()
{
    vector<long> lVec1, lVec2;         // Zwei leeren Vektor anlegen

    // 1. Vektor mit den ersten 20 Fibonacci-Zahlen fllen:
    generate_n( back_inserter(lVec1), 20, Fibonacci() );

    cout << "Die ersten 20 Fibonacci-Zahlen: " << endl;
    for_each( lVec1.begin(), lVec1.end(), print);
    cout << endl;

    // Nur gerade Zahlen kopieren, d.h. beim Kopieren die ungeraden entfernen:
    remove_copy_if( lVec1.begin(), lVec1.end(),  // Quellbereich
                    back_inserter(lVec2),        // Zielbereich
                    odd);                        // Bedingung

    cout << "\nDavon sind gerade:" << endl;
    for_each( lVec2.begin(), lVec2.end(), print); 
    cout << endl;

    // Im ersten Vektor die geraden Zahlen lschen:
    vector<long>::iterator new_last =            // Neues Ende
       remove_if( lVec1.begin(), lVec1.end(),    // Bereich
                  even);                         // Bedingung

    cout << "\nDie ungeraden Zahlen:" << endl;
    for_each( lVec1.begin(), new_last, print); 
    cout << endl;

    cout << "\nDie restlichen Elemente im ersten Vektor:\n";
    for_each( new_last, lVec1.end(), print);
    cout << "\nDie werden jetzt geloescht!" << endl;
    lVec1.erase( new_last, lVec1.end());

    cout << "Anzahl ungerader Zahlen: " << lVec1.size() << endl;
    return 0;
}

/* Ausgabe:

Die ersten 20 Fibonacci-Zahlen:
       1       2       3       5       8      13      21      34      55      89
     144     233     377     610     987    1597    2584    4181    6765   10946

Davon sind gerade:
       2       8      34     144     610    2584   10946

Die ungeraden Zahlen:
       1       3       5      13      21      55      89     233     377     987
    1597    4181    6765

Die restlichen Elemente im ersten Vektor:
     610     987    1597    2584    4181    6765   10946
Die werden jetzt geloescht!
Anzahl ungerader Zahlen: 13

*/
