// mutations.cpp: Beispiele zu den mutierenden Algorithmen.
// ---------------------------------------------------------------
#include <iostream>
#include <deque>            // Definition des deque-Templates
#include <algorithm>        // Definition der Algorithmen-Templates
#include <iterator>
using namespace std;

void print( const deque<int>& q)
{
   copy(q.begin(), q.end(), ostream_iterator<int>(cout,"  "));
   cout << endl;
}

int main()
{
    deque<int> iDeq;                      // Leere Queue
    for( int i = 1; i <= 5; ++i)
        iDeq.push_front(i);               // 5 4 3 2 1
    cout << "Die Elemente:               ";  print( iDeq);

    // Reihenfolge umdrehen:
    reverse( iDeq.begin(), iDeq.end());   // 1 2 3 4 5
    cout << "Nach reverse():             ";  print( iDeq);

    // Rotation nach links um 2 Stellen:
    rotate( iDeq.begin(), iDeq.begin()+2, iDeq.end());
                                          // 3 4 5 1 2
    cout << "Nach rotate() um 2 Stellen: ";  print( iDeq);

    // Zufllige Reihenfolge:
    random_shuffle( iDeq.begin(), iDeq.end());
    cout << "Nach random_shuffle():      ";  print( iDeq);

    // Zerlegen in Zahlen < 3 und >= 3:
    partition(iDeq.begin(), iDeq.end(),      // Prdikat als
              [](int a) { return a < 3; });  // Lambda.
    cout << "Nach partition() mit < 3:   ";  print( iDeq);

    sort( iDeq.begin(), iDeq.end());       // Wieder sortiert
    iDeq.pop_back(); iDeq.pop_back();      // und 4 u. 5 lschen
    // Alle Permutationen von 1 2 3 (3! = 9 Permutationen)
    cout << "Alle Permutationen von 1 2 3:" << endl;
    do  print( iDeq);         // 1 2 3,  1 3 2,  2 1 3, ... 
    while( next_permutation( iDeq.begin(), iDeq.end()) );

    return 0;
}

/* Beispiel-Ausgabe:

Die Elemente:               5  4  3  2  1
Nach reverse():             1  2  3  4  5
Nach rotate() um 2 Stellen: 3  4  5  1  2
Nach random_shuffle():      2  4  1  5  3
Nach partition() mit < 3:   2  1  4  5  3
Alle Permutationen von 1 2 3:
1  2  3
1  3  2
2  1  3
2  3  1
3  1  2
3  2  1

*/
