// sort.cpp: Beispiele zu den Sortier-Algorithmen.
// ---------------------------------------------------------------
#include <iostream>
#include <algorithm>       // Definition der Algorithmen-Templates
using namespace std;

void pr(int i) { cout.width(4); cout << i;  }
void print(int v[], int len)
{  for_each( v, v+len, pr);  cout << endl; } 

bool odd(int a) { return a%2 != 0; }
// "gerade" kleiner als "ungerade":
bool evenCmp(int a, int b) { return !odd(a) && odd(b); }  // oder: abs(a)%2 < abs(b)%2;

int main()
{
    // Die zu sortierenden Zahlen:   
    int zahlen[] = { 4, 1, 0, -5, 3, -1, 2, -3 };
    int len = sizeof(zahlen)/sizeof(int);       // Anzahl Elemente im Vektor

    cout << "Unsortiert:                   ";  print(zahlen,len);

    // Das zweit-kleinste Element: 
    nth_element( zahlen, zahlen+1, zahlen+len);
    cout << "Das zweit-kleinste Element: " << zahlen[1] << endl;
    cout << "Ergebnis von nth_element():   ";  print(zahlen, len);

    // "normale" Sortierung mit Operator <
    sort( zahlen, zahlen+len);
    cout << "Sortiert mit Operator <       ";  print(zahlen, len);

    // Gerade Zahlen vor ungeraden anordnen. Mit stabel_sort()
    // bleiben die geraden (ungeraden) Zahlen sortiert:
    stable_sort( zahlen, zahlen+len, evenCmp);
    cout << "Gerade Zahlen vor ungeraden:   ";  print(zahlen, len);

    // Position des ersten ungeraden Elementes:
    int *pos = find_if(zahlen, zahlen+len, odd);
    cout << "Erste ungerade Zahl:  " << *pos << endl;

    // Die zwei sortierten Bereiche wieder mischen:
    inplace_merge(zahlen, pos, zahlen+len);
    cout << "Ergebnis von inplace_merge(): ";  print(zahlen, len);

    return 0;
}
/* Ausgabe:

Unsortiert:                      4   1   0  -5   3  -1   2  -3
Das zweit-kleinste Element: -3
Ergebnis von nth_element():     -5  -3  -1   0   2   1   3   4
Sortiert mit Operator <         -5  -3  -1   0   1   2   3   4
Gerade Zahlen vor ungerade:      0   2   4  -5  -3  -1   1   3
Erste ungerade Zahl:  -5
Ergebnis von inplace_merge():   -5  -3  -1   0   1   2   3   4
*/

