// polynom.h:    Darstellung von Polynomen
//-----------------------------------------------------
#include <valarray>
using namespace std;

template <class T>
class Polynom
{
  private:
    valarray<T> koeff;

  public:
    Polynom(size_t grad = 0) : koeff(grad+1){} 
    Polynom(const T& a, size_t grad) : koeff(a, grad+1){}
    Polynom(const T* pk, size_t grad):koeff(pk, grad+1){} 
                       // Hier ohne Bereichsberprfung:
    T& operator[](int i) { return koeff[i];}
    T  operator[](int i) const { return koeff[i];} 
    int grad() const;              // Grad des Polynoms
    size_t size() const{ return koeff.size();} 
    Polynom<T>& resize(size_t n);  

    Polynom<T>& operator=(const Polynom<T>& a);
    Polynom<T>& operator=(const T& a); 

    T operator()(const T& x);      // Funktionswert

    Polynom<T>& operator+=(const Polynom<T>& a);
    Polynom<T>& operator*=(const Polynom<T>& a);
};

template <class T>
int Polynom<T>::grad() const
{ 
  // Suche hoechsten von 0 verschiedenen Koeffizienten
  for( int i=(int)koeff.size()-1; i >= 0; i--)
    if(koeff[i] != T(0)) 
        return i;
  return 0;
}

template <class T>
Polynom<T>& Polynom<T>::resize(size_t n)   // Grad n
{
  if( n+1 != koeff.size())
  {
    valarray<T> temp(koeff);
    koeff.resize(n+1);                   // Alle Werte 0.
    size_t min = n+1 < temp.size() ? n+1 : temp.size();
    for (size_t i = 0; i < min; ++i)     // Alte Werte
        koeff[i] = temp[i];              // bernehmen.
  }
  return *this;
}

template <class T>
Polynom<T>& Polynom<T>::operator=(const Polynom<T>& a)
{
   koeff = a.koeff;       // Mit Anpassung der Lnge.
   return *this;
}

template <class T>
Polynom<T>& Polynom<T>::operator=(const T& a) 
{
   koeff = a;
   return *this;
}

// Funktionswert des Polynoms nach dem Horner-Schema
template <class T>
T Polynom<T>::operator()(const T& z)
{
  T sum(0);
  for(int i = grad(); i >= 0 ; i--)
     sum = sum * z + koeff[i];
  return sum;
}

template <class T>
Polynom<T>& Polynom<T>::operator+=(const Polynom<T>& a)
{ 
   size_t s1 = size(), s2 = a.size();
   Polynom<T> temp(a);
                                    // Gleiche Lngen?
   if( s1 < s2)     resize(s2-1);       // Grad s2-1
   else if(s1 > s2) temp.resize(s1-1);  // Grad s1-1

   koeff += temp.koeff;
   return *this;
}

template <class T>
Polynom<T>& Polynom<T>::operator*=(const Polynom<T>& a)
{  
    int n = grad(), m = a.grad(), i, j;
    Polynom<T>  res(T(0),n+m);

    for( i = 0; i <= n; ++i)
      for( j = 0; j <= m; ++j)   
        res[i+j] += koeff[i]* a[j];
/*
    for( i = 0; i <= n; i++)
      for( j = 0; j <= i && j <= m; j++)   
        res[i] += koeff[i-j]* a[j];
    for(i= n+1; i <= n+m; i++)
      for(j=i; j <= n+m; j++)
          if(j >= m)  res[i] += a[m-j+i]*koeff[j-m];
*/
    return (*this = res);
}

// Globale Operatorfunktionen:
template<class T>
Polynom<T> operator+(const Polynom<T>& a, 
                     const Polynom<T>& b)
{   
    Polynom<T> c(a);
    c += b;
    return c;
}

template<class T>
Polynom<T> operator*(const Polynom<T>& a, 
                     const Polynom<T>& b)
{   
    Polynom<T> c(a);
    c *= b;
    return c;
}
