// ---------------------------------------------------------
// VekQueue.h
// Definition des Klassen-Templates VekQueue<T>
// zur Darstellung eines Vektors von Queues.
// ---------------------------------------------------------
#ifndef VEKQUEUE_H
#define VEKQUEUE_H

#include <vector>
#include <queue>
#include <cstdlib>            // Für srand(), rand()
#include <ctime>              // Für time()
using namespace std;

template <class T>
class VekQueue
{
  private: 
    vector< queue<T> > v;
    size_t sz;                // Anzahl Queues

  public:
    VekQueue(size_t n);       // Konstruktor

    size_t size() const;      // Anzahl aller Elemente
    size_t size(int i) const  // Anzahl in der i-ten Queue
    { return v[i].size(); }

    bool empty() const      { return size() == 0; }
    bool empty(int i) const { return size(i) == 0; }

    void push(const T& a);    // Hot-Potato-Algorithmus.

    const T& pop();           // Entnimmt das Element am Anfang
                              // einer zufällig gewählten Queue.
    const T& pop(int i);      // Entnimmt das Element am Anfang
                              // der i-ten Queue
};

template <class T>
VekQueue<T>::VekQueue( size_t n )
{ 
    if(n > 0)
      v.resize(n);            // Platz für n Queues

    sz = n;
    srand((unsigned int)time(NULL));   // Zufallszahlen initialisieren
}

template <class T>            // Anzahl aller Elemente 
size_t VekQueue<T>::size() const
{
    size_t count = 0;
    for( int i=0; i < sz; ++i)
      count += v[i].size();

    return count;
}

template <class T>            // Fügt a in kuerzeste Queue ein
void VekQueue<T>::push(const T& a)
{
    size_t small = 0;         // Kürzeste Queue bestimmen
    for(size_t i = 0; i < sz; i++)
      if( v[i].size() < v[small].size() )
          small = i;

    v[small].push(a);         // und dort einfügen.
}

template <class T>            // Element aus zufaellig 
const T& VekQueue<T>::pop()   // ausgewählter Queue entnehmen. 
{
   static T temp;
   int i, i0;

   temp = T();
   i = i0 = rand() % sz;
   do
   {
     if(!v[i].empty())          // Falls i-te Queue nicht leer:
     {                          // Element am Anfang der Queue
       temp = v[i].front();     // hervorholen
       v[i].pop();              // und löschen.
       break;
     }
     i = (i+1) % sz;            // Sonst nächste Queue.
   }
   while( i != i0);

   return temp;
}

template <class T>               // Element aus der i-ten Queue
const T& VekQueue<T>::pop(int i) // entnehmen.
{
   static T temp;
   temp = T();

   if( i >= 0 && i < (int)sz     // Falls gültiger Index
              && !v[i].empty())  // und nicht leer:
   {                             // Element am Anfang der Queue
       temp = v[i].front();      // hervorholen
       v[i].pop();               // und löschen.
   }

   return temp;
}

#endif   // VEKQUEUE_H
