// ----------------------------------------------------
// index.cpp: Implementiert die Methoden der Klassen 
//            IndexEntry, IndexFile und IndexFileSystem
// ----------------------------------------------------

#include "index.h"

fstream& IndexEntry::write(fstream& ind) const
{
   ind.write((char*)&key, sizeof(key) );
   ind.write((char*)&recNr, sizeof(recNr) );

   return ind;
}

fstream& IndexEntry::read(fstream& ind)
{
   ind.read((char*)&key, sizeof(key) );
   ind.read((char*)&recNr, sizeof(recNr)); 

   return ind;
}

fstream& IndexEntry::write_at(fstream& ind, long pos) const
{
   ind.seekp(pos);
   ind.write((char*)&key, sizeof(key) );
   ind.write((char*)&recNr, sizeof(recNr) );

   return ind;
}

fstream& IndexEntry::read_at(fstream& ind, long pos)
{
   ind.seekg(pos);
   ind.read((char*)&key, sizeof(key) );
   ind.read((char*)&recNr, sizeof(recNr)); 

   return ind;
}

// ---------------------------------------------------
// Methoden der Klasse IndexFile

IndexFile::IndexFile(const string& file)
{ 
   ios::openmode mode =  ios::in | ios::out | ios::binary;

   index.open( file, mode);  // ffnen, falls Datei schon existiert.

   if(!index)                // Falls Datei nicht vorhanden
   {
      index.clear();
      mode |= ios::trunc;
      index.open( file, mode);
      if(!index)
         throw open_error(name);
   }
   name = file;
}

void IndexFile::display() 
{
    IndexEntry entry;

    index.seekg(0L);

    if(!index)
       throw read_error("IndexFile: Positionieren des get-Zeigers");

    cout << "\nDer Index: " << endl;
    while( true)
    {
       if( !entry.read(index))
           break;
       entry.display();
    }
    if( !index.eof())
        throw read_error(name);

    index.clear(); 
}

long IndexFile::search(long k)
{
   IndexEntry entry;
   long key;
   long mid, begin = 0, end;        // Datensatznummern.
   int  size = entry.recordSize();  // Lnge eines Index-Eintrags.

   index.clear();
   index.seekg(0L, ios::end);
   end = (long)index.tellg() / size;

   if(!index)
         throw read_error(name);

   if( end == 0)
       return -1;

   end -= 1;                      // Position des letzten Eintrags

   while( begin < end )
   {
       mid = (begin + end +1)/2 ;

       entry.read_at(index, mid*size);
       
       if(!index)
         throw read_error(name);

       key = entry.getKey();
       if( k < key)
           end = mid - 1;
       else 
           begin = mid;
   }

   entry.read_at(index, begin * size);
   if(!index)
         throw read_error(name);

   if ( k == entry.getKey() )    // Schlssel gefunden?
       return begin * size;
   else
       return -1;
}

void IndexFile::insert(long k, long n)
{
    IndexEntry entry;
    int size = entry.recordSize();  // Lnge eines Index-Eintrags.

    index.clear(); 
    index.seekg(0, ios::end);
    long nr = (long)index.tellg();  // Dateilnge,
                                    // 0, falls Datei leer.
    if(!index)
        throw read_error(name);

    nr-= size;                      // Letzter Eintrag.
    bool found = false;
    while( nr >= 0 && !found )      // Position zum Einfgen suchen
    {
       if(!entry.read_at(index, nr))
          throw read_error(name);

       if( k < entry.getKey())      // Shiften, um Platz zu machen.
       {
          entry.write_at(index, nr + size);
          nr -= size;
       }
       else                         // Gefunden
       {
         found = true;
       }
    }

    entry.setKey(k);  entry.setPos(n);   // Einfuegen.
    entry.write_at(index, nr + size);

    if(!index)
        throw write_error(name);
}


void IndexFile::retrieve( IndexEntry& entry, long pos )
{
    index.clear();
    if( !entry.read_at(index, pos))
        throw read_error(name);
}


// ---------------------------------------------------
//  Methoden der Klasse IndexFileSystem.

bool IndexFileSystem::insert( const Konto& kto)
{
  if(index.search(kto.getNr()) == -1)       // Keine mehrfachen Eintrge.
  {
    long pos = konten.append(kto);          // In Primrdatei einfgen.
    if(pos != -1)
    {
      index.insert(kto.getNr(), pos);       // Im Index einfgen.
      return true;
    }
  }
  return false;
}

Konto* IndexFileSystem::retrieve(long key )
{
                                 // Im Index Datensatzadresse holen:
   long pos = index.search(key); // Byte-Offset des Index-Eintrags.

   if( pos == -1 )               // Kontonummer nicht vorhanden.
     return NULL;
   else                          // Falls Kontonummer vorhanden:
   {
      IndexEntry entry;
      index.retrieve(entry, pos);     // Index-Eintrag lesen.
                                      // Datensatz holen: 
      return konten.retrieve( entry.getPos() ); 
   }
}
