// -----------------------------------------------------------
// index.h:  Enthlt die Definition der Klassen
//           IndexEntry zur Darstellung eines Indexeintrags, 
//           IndexFile  zur Darstellung einer Index-Datei und
//           IndexFileSystem zur Darstellung eines
//                           Index-Datei-Systems.
// -----------------------------------------------------------
#ifndef INDEX_H
#define INDEX_H

#include <fstream>
#include <iostream>
#include <string>
#include "konto.h"
using namespace std;

#include "exceptio.h"   // Definition der Fehlerklasse 
                        // open_error, read_error, write_error

class IndexEntry
{
  private:
    long key;
    long recNr;

  public:
    IndexEntry(long k=0L, long n=0L) { key = k; recNr = n; }

    void setKey(long k) { key = k; }
    long getKey() const { return key; }
    void setPos(long n) { recNr = n; }
    long getPos() const { return recNr; }

    int recordSize() const
    { return sizeof(key) + sizeof(recNr); }

    fstream& write(fstream& ind ) const;
    fstream& read( fstream& ind);

    fstream& write_at(fstream& ind, long pos ) const;
    fstream& read_at( fstream& ind, long pos);

    void display()
    {
       cout << "Kto-Nr: " << key
            << "   Pos: " << recNr << endl;
    }
};

// ----------------------------------------------------------  
// Definition der Klasse IndexFile:

class IndexFile
{
  private:
    fstream index;
    string  name;              // Name der Indexdatei

  public:
    IndexFile(const string& s);                   // Mgliche Exception: open_error
    ~IndexFile() { index.close(); }

    void insert(long key, long pos);              // Mgliche Exceptions: read_error, write_error
    long search(long key);                        // Mgliche Exception:  read_error
    void retrieve(IndexEntry& entry, long pos);   // Mgliche Exception:  read_error      

    void display();                               // Mgliche Exception:  read_error
};

// ----------------------------------------------------------  
// Definition der Klasse IndexFileSystem:

class IndexFileSystem
{
   private:
     KontoFile konten;        // Primrdatei
     IndexFile index;         // Indexdatei

   public:
     IndexFileSystem(const string& s) 
     : konten(s + ".prim"), index(s + ".ind")
     { }

     bool   insert  ( const Konto& kto );
     Konto* retrieve( long key );

     void displayIndexFile() { index.display(); }
     void displayKontoFile() { konten.display(); }
};

#endif
