// unordered_set.cpp
// Demonstriert die Verwendung des Templates unordered_set<T> 
// ---------------------------------------------------------- 
#include <unordered_set>
#include <string>
#include <iostream>
using namespace std;

// Klassen Customer und hash<Customer>
class Customer
{
  private:
    int id; string name;   // Schlssel und Kundename 
  public:
    Customer(int i=0, const string& s="X") : id(i), name(s) {}

    // Zugriffsmethoden getID() usw. und Operatorfunktionen:
    int getID() const { return id; }

    friend bool operator==(const Customer& a,
                           const Customer& b)
    {   return a.id == b.id;  }

    friend ostream& operator<<(ostream& os, const Customer& ct)
    { 
        os << "ID = " << ct.id << ", Name = "<< ct.name;
        return os;
    } 
}; 

namespace std     // Spezialisierung im selben Namensbereich,
{                 // in dem auch hash<T> definiert ist.
  template<> struct hash<Customer>
  {
     size_t operator()(const Customer& ct) const
     {
        hash<int> int_hash;  
        return int_hash(ct.getID());
     }
  };
}

typedef unordered_set<Customer> HashTable;

int main()
{
    HashTable hashtable;
    hash<Customer> hash = hashtable.hash_function(); 

    hashtable.max_load_factor(2);   // Max. Belegungsfaktor: 2 
                                    // Zu Testzwecken einfgen:
    for( int i= 1; i <= 15; ++i)
       hashtable.emplace(12 + i);

    //for( auto &cust : hashtable)
    //    cout << cust << endl;
    //cin.get();

    // Bucketweise ausgeben: 
    for(size_t i = 0; i < hashtable.bucket_count(); i++) 
    {
      cout << "Bucketnummer: " << i << endl;
      if( hashtable.bucket_size(i) > 0)
      {
        auto loc_it = hashtable.begin(i);     // local_iterator
        while(loc_it != hashtable.end(i)) 
              cout  << "Kunde: " << *loc_it++  << endl;
      }
    }

    cout << "\nAnzahl Buckets: " 
         << hashtable.bucket_count()  << endl;

                                             // Bucket suchen:
    cout <<  "Kunde mit Kundennummer 21 suchen. "  << endl; 

    auto it = hashtable.find(21);

    cout << "Kunde: " << *it << endl;
    size_t nr = hashtable.bucket(it->getID());
    cout << "Bucketnummer: " << nr << endl;

    cout << "Anzahl Objekte im Bucket: " 
         << hashtable.bucket_size(nr) << endl;

    cout << "Hash-Wert: " << hash(*it) << endl;

    return 0;
}
